###############################################################################
# Msys tools logging module
#
# This modules provides logging facilities to Msys and City packages.
#
# Return values:
###############################################################################
package MsysLog;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

use      Sys::Hostname;
use      Sys::Syslog;

require Exporter;

@ISA = qw(Exporter AutoLoader);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.
@EXPORT = qw(
	
);
$VERSION = '0.01';

# Preloaded methods go here.

my $TRUE    = (1 == 1);
my $FALSE   = (1 == 0);
my $CONFOBJ = undef;

my $MAX_SYSLOG_LVL = 7;
my %LEVEL_NAME = (
     0 => 'emerg',
     1 => 'alert',
     2 => 'crit',
     3 => 'err',
     4 => 'warning',
     5 => 'notice',
     6 => 'info',
     7 => 'debug',
	 # The rest support higher debug logging to files, _not_ SYSLOG
	 8 => 'debug',
	 9 => 'debug');

sub new {
    my $pkg = shift;
    my $class = ref($pkg) || $pkg;

    return($CONFOBJ) if (defined $CONFOBJ);

    my %args = @_;
    $CONFOBJ = bless {
	    PROGRAM       => $args{PROGRAM}       || _get_my_prog('short'),
		SYSLOGFAC     => $args{SYSLOGFAC}     || $FALSE,
		SYSLOGOPT     => $args{SYSLOGOPT}     || $FALSE,
		FILELOG       => $args{FILELOG}       || $FALSE,
		DEFAULT_LEVEL => $args{DEFAULT_LEVEL} || 6, # -> info
		FILELOG_LEVEL => $args{FILELOG_LEVEL} || 9999,
		HOSTNAME      => _get_my_host('short'),
    };

	if ($CONFOBJ->{SYSLOGFAC}) {
  	    $CONFOBJ->{SYSLOGOPT} ||= 'cons, pid';
	}

	if ( $CONFOBJ->{FILELOG} =~ /^default$/i ) {
        my $defdir  = '/var/log';
        my $deffile = "/var/log/$CONFOBJ->{PROGRAM}.log";
        my $altfile = "/tmp/$CONFOBJ->{PROGRAM}.log";
        if ( (-e $deffile && -w $deffile) || -w $defdir ) {
		    $CONFOBJ->{FILELOG} = $deffile;
	    } else {
		    $CONFOBJ->{FILELOG} = $altfile;
        }
	}

    $CONFOBJ->_initialize();
    return($CONFOBJ);
}


sub _initialize {
    my $self = shift;

    if ($self->{SYSLOGFAC}) {
	    if (defined(&Sys::Syslog::setlogsock)  &&
		    defined(&Sys::Syslog::_PATH_LOG) ) { # Net::Daemon::Log
            &Sys::Syslog::setlogsock('unix');
	    }
        openlog($self->{PROGRAM}, $self->{SYSLOGOPT}, $self->{SYSLOGFAC});
    }

	if ($self->{FILELOG}) {
        if ($self->{FILELOG} =~ /^stdout$/i) {
		    *STDLOG = *STDOUT;
		} elsif ($self->{FILELOG} =~ /^stderr$/i) {
		    *STDLOG = *STDERR;
		} else {
            open(STDLOG, ">>$self->{FILELOG}")
			    or _fatal("Error on log open '$self->{FILELOG}', rc=$?");
		    my $oldfh = select STDLOG;
			$| = 1; # autoflush;
			select $oldfh;
        }
    }
}

sub Log ($$$;@) {
    my ($self, $level, $format, @args) = @_;

    my $message = sprintf($format, @args);

	$level ||= $self->{DEFAULT_LEVEL};
    my $prtlevel = $LEVEL_NAME{$level} || $level; # Translate is number

    if ( $self->{FILELOG} && $level <= $self->{FILELOG_LEVEL} ) {
	    print STDLOG _timeprint(), ' ', $self->{HOSTNAME},
		      " $self->{PROGRAM}\[$$]: $message\n";
    }
	if ($self->{SYSLOGFAC} && $level <= $MAX_SYSLOG_LVL) {
	    syslog($prtlevel, $message);
    }
}

sub Debug ($$;@) {
    my ($self, $format) = (shift, shift);
	if (!ref($self) || $self->{FILELOG_LEVEL}) {
	    $self->Log(7, $format, @_);
	}
}

sub Error ($$;@) {
    my ($self, $format) = (shift, shift);
	$self->Log(3, $format, @_);
}

sub Fatal ($$;@) {
    my ($self, $format) = (shift, shift);
	my $message = sprintf($format, @_);
	$self->Log(3, $message);
	die $message;
}

sub _get_my_host {
    my $mode = shift;
    my $host = hostname();
    chomp $host;
    if ($mode eq 'short') {
        while ($host =~ m#\.#) { $host = "$`"; }
    }
    return($host);
}

sub _get_my_prog {
    my $prog = $0; 
	while ($prog =~ m#/#) { $prog = "$'"; }
	return($prog);
}


sub _timestamp {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
    $mon++;
    if ($mon  < 10) { $mon  = '0'.$mon;  }
    if ($mday < 10) { $mday = '0'.$mday; }
    if ($hour < 10) { $hour = '0'.$hour; }
    if ($min  < 10) { $min  = '0'.$min;  }
    if ($sec  < 10) { $sec  = '0'.$sec;  }
    return("$year.$mon.$mday.$hour.$min.$sec");
}

sub _timeprint {
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
    my @MONTHS = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
    $year += 1900;
    my $month = $mon + 1;
    if ($month < 10) { $mon  = '0'.$mon;  }
    if ($mday  < 10) { $mday = '0'.$mday; }
    if ($hour  < 10) { $hour = '0'.$hour; }
    if ($min   < 10) { $min  = '0'.$min;  }
    if ($sec   < 10) { $sec  = '0'.$sec;  }
    my $montxt = $MONTHS[$mon];
    return("$montxt $mday $hour:$min:$sec");
}
sub _fatal {
    print shift;
    exit(1);
}

sub DESTROY {
    my $self = shift;
	if ($self->{FILELOG} &&
        $self->{FILELOG} !~ /^stderr$/i &&
        $self->{FILELOG} !~ /^stdout$/i) {
		close(STDLOG);
	}
    if ($self->{SYSLOGFAC}) {
        closelog();
	}
}

1;
# Autoload methods go after =cut, and are processed by the autosplit program.

__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

MsysLog - Module for logging to Syslog and/or a file.

=head1 SYNOPSIS

 use MsysLog;
 $log = new MsysLog( PROGRAM   => 'myprog',
                     SYSLOGFAC => 'local2',
                     FILELOG   => '/var/log/myprog.log' );

 $log->Log('debug', 'Starting myprog');
 $log->Error('Problem!');

=head1 DESCRIPTION

This module provides a generic interface for logging to syslog and/or
a file (both can occur at the same time).

To initialize logging use the following Perl object samantics and named
arguments:

$log = new Msyslog (

PROGRAM  => '<program_name>',
   Optional argument to specify the program name appearing in logs.
   If not specified a default is obtained from $0.

SYSLOGFAC => '<facility_name>',
   Optional argument to specify a syslog facility. See syslog(3).
   Only when this argument is provided is syslog'ing enabled.

SYSLOGOPT => '<alternate_options>',
   Optional argument to specify syslog options. See syslog(3) "option".
   This argument does NOT enable syslog'ing.
   If not specified and syslog'ing, the default is 'cons, pid'.

FILELOG => 'default|stdout|stderr|<some_file>',
   Optional argument to specify a file to print logging messages to.
   If 'stdout' is specified, logging is to STDOUT.
   If 'stderr' is specified, logging is to STDERR.
   If 'default' is specified, logging is to /var/log/<program_name>.log
      if the file can be written to (or created)
	  else logging is to /tmp/<program_name>.log
   All other values are assumed to be an absolute file name.

DEFAULT_LEVEL => '<number>',
   Optional argument to specify a default message log level to be used
      for messages that don't specify a level.
   If not specified, the default is 6 => info.
   Valid <number> values are 1 thru 9.

FILELOG_LEVEL => '<number>',
   Optional argument to specify a minimum *FILE* log level.
   Levels 1-7 can be written to both syslog and files, but levels above 7
      will only be written to the specified FILELOG when a FILELOG_LEVEL
	  greater than 7 is specified.
   This argument allows applications to incorporate increasingly detailed
      debug logging messages that get thrown out unless a runtime debug
	  level is set.


To log messages use one of the following:

 $log->Log(<level>, <format> [, @args]);
 $log->Debug(<format> [, @args]);    # <level> = 7
 $log->Error(<format> [, @args]);    # <level> = 3
 $log->Fatal(<format> [, @args]);    # <level> = 2

Where:
   <level> is a number in the range 0 to 9
           or a syslog_level: emerg|alert|crit|err|warning|notice|info|debug
   <format> is a sprintf format (see perldoc for function sprintf),
   @args are optional arguments to the sprintf function,

=head1 AUTHOR

John-Paul Navarro, navarro@mcs.anl.gov

=head1 SEE ALSO

syslog(2), syslog(3), perl Sys::Syslog module.

=cut
